function [tout,yout,step]=int_walker_turbo(t0,tfinal,...
y0,tol,m,g,R,sgam,cgam,Izz,p12x,p23x,pc1,pc2,pc3,pcs,p1k,...
lt,ls,etak,etat,etaf,pc2lock);

global K KK 

%       C.B. Moler, 3-25-87, 10-5-91, 6-3-93.
%       Copyright (c) 1984-93 by The MathWorks, Inc.
%       Modified for walking simulation
%       by Mariano Garcia -Summer, Fall 1994

%%%%%%%%%%%%%%%%%%%%%%%%%% Program begins  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Initialization
% The Fehlberg coefficients:
beta  = [ [    1      0      0     0      0    0]/4
          [    3      9      0     0      0    0]/32
          [ 1932  -7200   7296     0      0    0]/2197
          [ 8341 -32832  29440  -845      0    0]/4104
          [-6080  41040 -28352  9295  -5643    0]/20520 ]';
gamma = [ [902880  0  3953664  3855735  -1371249  277020]/7618050
          [ -2090  0    22528    21970    -15048  -27360]/752400 ]';
pow = 1/5;
f = zeros(length(y0),6);

if nargin < 5, tol = 0.001; end
t = t0;
w=1;
step(1,:)=y0;

hmax = (tfinal - t)/16;
h = hmax/8;
y = y0(:);
chunk = 128;
tout = zeros(chunk,1);
yout = zeros(chunk,length(y));
k=1;
tout(k) = t;
yout(k,:) = y.';
tau = tol * max(norm(y, 'inf'), 1);
clf reset;
axis([-1 1 0 1]);
axis equal;

% Get coordinates for graphics
% cfx,cfy are coordinates of center of stance foot in a world fixed frame
% hip, knee, and end are coordinates of hip, swing knee, and end foot center

cfx=-R*y(1);
cfy=R;
cfhlength=lt*cos(etat)+(ls-R)*cos(etaf);
hipx=cfx-cfhlength*sin(y(1));
hipy=cfy+cfhlength*cos(y(1));
kneex=hipx-lt*sin(pi-y(1)-y(2));
kneey=hipy-lt*cos(pi-y(1)-y(2));
endx=kneex-(ls-R)*sin(pi-y(1)-y(2)-y(3));
endy=kneey-(ls-R)*cos(pi-y(1)-y(2)-y(3));

%PLot graphics initially

walker_data=[cfx,cfy;hipx,hipy;kneex,kneey;endx,endy];
walker_bits=line('xdata',walker_data(:,1),'ydata',walker_data(:,2)...
,'erasemode','xor');
drawnow;


%%%%%%%%%%%%%%%%%%%%%  Main Loop Begins  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The main loop.  Assume that we are in three link mode before kneestrike
while (t < tfinal)


   % assume three link mode for starters
%%%%%%%%%%%%%%%%%%%%% Begin Three Link Loop %%%%%%%%%%%%%%%%%%%%%%%%%%%%%


	while  (y(3)+h*y(6) < etak) & t < tfinal      
         % Compute the slopes
   f(:,1) = yderivs_new_not(y,K);
   for j = 1:5
      f(:,j+1) = yderivs_new_not(y+h*f*beta(:,j),K);
   end

% Estimate the error and the acceptable error
   delta = norm(h*f*gamma(:,2),'inf');
   tau = tol*max(norm(y,'inf'),1.0);

        % Update the solution only if the error is acceptable
   if delta <= tau
      t = t + h;
      y = y + h*f*gamma(:,1);
      k = k+1;
      if k > length(tout)
         tout = [tout; zeros(chunk,1)];
         yout = [yout; zeros(chunk,length(y))];
      end
      tout(k) = t;
      yout(k,:) = y.';


	   %Update Graphics

	cfx=-R*y(1);
	cfy=R;
	hipx=cfx-cfhlength*sin(y(1));
	hipy=cfy+cfhlength*cos(y(1));
	kneex=hipx-lt*sin(pi-y(1)-y(2));
	kneey=hipy-lt*cos(pi-y(1)-y(2));
	endx=kneex-(ls-R)*sin(pi-y(1)-y(2)-y(3));
	endy=kneey-(ls-R)*cos(pi-y(1)-y(2)-y(3));
 
walker_data=[cfx,cfy;hipx,hipy;kneex,kneey;endx,endy];
set(walker_bits,'xdata',walker_data(:,1),'ydata',walker_data(:,2));

	drawnow;

	end;

        % Update the step size

      if delta ~= 0.0
      h = min(hmax, 0.8*h*(tau/delta)^pow);
     end

    horig=h;
   end;
%%%%%%%%%%%%%% End THree Link Loop %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% now we are less than h seconds away from strike
% do Newton's method to get to kneestrike
% assume we want to find the right h to get us right to kneestrike
% The swing shank is at etak-y(3) radians away from strike
% and moving with angular velocity y(6) rad/sec.

  if t<tfinal & abs(y(6))>tol
	q=0;

	while q<4
	  h=(etak-y(3))/y(6);

   % Compute the slopes
   f(:,1) = yderivs_new_not(y,K);
   for j = 1:5
      f(:,j+1) = yderivs_new_not(y+h*f*beta(:,j),K);
   end

        % Update the solution
      t = t + h;
      y = y + h*f*gamma(:,1);
      k = k+1;
      if k > length(tout)
         tout = [tout; zeros(chunk,1)];
         yout = [yout; zeros(chunk,length(y))];
      end
      tout(k) = t;
      yout(k,:) = y.';

	  q=q+1;
end;

%Update Graphics

	cfx=-R*y(1);
	cfy=R;
	hipx=cfx-cfhlength*sin(y(1));
	hipy=cfy+cfhlength*cos(y(1));
	kneex=hipx-lt*sin(pi-y(1)-y(2));
	kneey=hipy-lt*cos(pi-y(1)-y(2));
	endx=kneex-(ls-R)*sin(pi-y(1)-y(2)-y(3));
	endy=kneey-(ls-R)*cos(pi-y(1)-y(2)-y(3));

walker_data=[cfx,cfy;hipx,hipy;kneex,kneey;endx,endy];
set(walker_bits,'xdata',walker_data(:,1),'ydata',walker_data(:,2));
	drawnow;


%%%%%%%%%%%%%%%%%%%%%%%%%%%% Kneestrike %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
disp('out of 3 link loop')

ynew = kneestrike_turbo(y,m,R,Izz,p12x,p23x,pc1,pc2,pc3);

  y = ynew';
  h=horig/6;
%store new y at same t
         k = k+1;

         if k > length(tout)
                tout = [tout; zeros(chunk,1)];
                yout = [yout; zeros(chunk,length(y))];
         end
         tout(k) = t;
         yout(k,:) = y.';

  disp('kneelocked')
  end;

%%%%%%%%%%%% Begin Two Link Loop %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
xxx=1;
if y(2)-(pi-etat-2*y(1))>0 xxx=0; end;
	while ((y(2)+h*y(5))<(pi-etat-2*y(1)-2*h*y(4))) & xxx==1
%after kneestrike but before heelstrike
 % Compute the slopes
   f(:,1) = yderivstwo_new_not(y,KK);
   for j = 1:5
      f(:,j+1) = yderivstwo_new_not(y+h*f*beta(:,j),KK);
   end
% Estimate the error and the acceptable error
   delta = norm(h*f*gamma(:,2),'inf');
   tau = tol*max(norm(y,'inf'),1.0);

        % Update the solution only if the error is acceptable
   if delta <= tau
      t = t + h;
      y = y + h*f*gamma(:,1);
      k = k+1;
      if k > length(tout)
         tout = [tout; zeros(chunk,1)];
         yout = [yout; zeros(chunk,length(y))];
      end
      tout(k) = t;
      yout(k,:) = y.';

         % Update the plot
		cfx=-R*y(1);
		cfy=R;
		hipx=cfx-cfhlength*sin(y(1));
		hipy=cfy+cfhlength*cos(y(1));
		kneex=hipx-lt*sin(pi-y(1)-y(2));
		kneey=hipy-lt*cos(pi-y(1)-y(2));
		endx=kneex-(ls-R)*sin(pi-y(1)-y(2)-y(3));
		endy=kneey-(ls-R)*cos(pi-y(1)-y(2)-y(3));

walker_data=[cfx,cfy;hipx,hipy;kneex,kneey;endx,endy];
	set(walker_bits,'xdata',walker_data(:,1),'ydata',walker_data(:,2));
		drawnow;

  	     end;

      % Update the step size
     	   if delta ~= 0.0
       	    h = min(hmax, 0.8*h*(tau/delta)^pow);
     	   end
  	 end;

       while ((y(2)+h*y(5))>=(pi-etat-2*y(1)-2*h*y(4))) & t < tfinal
%after kneestrike but before heelstrike
% Compute the slopes
   f(:,1) = yderivstwo_new_not(y,KK);
   for j = 1:5
      f(:,j+1) = yderivstwo_new_not(y+h*f*beta(:,j),KK);
   end

% Estimate the error and the acceptable error
   delta = norm(h*f*gamma(:,2),'inf');
   tau = tol*max(norm(y,'inf'),1.0);

        % Update the solution only if the error is acceptable
   if delta <= tau
      t = t + h;
      y = y + h*f*gamma(:,1);
      k = k+1;
      if k > length(tout)
         tout = [tout; zeros(chunk,1)];
         yout = [yout; zeros(chunk,length(y))];
      end
      tout(k) = t;
      yout(k,:) = y.';

         % Update the plot
                cfx=-R*y(1);
                cfy=R;
                hipx=cfx-cfhlength*sin(y(1));
                hipy=cfy+cfhlength*cos(y(1));
                kneex=hipx-lt*sin(pi-y(1)-y(2));
                kneey=hipy-lt*cos(pi-y(1)-y(2));
                endx=kneex-(ls-R)*sin(pi-y(1)-y(2)-y(3));
                endy=kneey-(ls-R)*cos(pi-y(1)-y(2)-y(3));

walker_data=[cfx,cfy;hipx,hipy;kneex,kneey;endx,endy];
        set(walker_bits,'xdata',walker_data(:,1),'ydata',walker_data(:,2));
                drawnow;
             end;

      % Update the step size
           if delta ~= 0.0
            h = min(hmax, 0.8*h*(tau/delta)^pow);
           end
         end;

%%%%%%%%%%%%%%%%%%%%%%% Out of Two Link Loop %%%%%%%%%%%%%%%%%%%%%%%%%%%
  if t<tfinal 
	horig=h;

q=0;
	while q<4
	h=(pi-etat-2*y(1)-y(2))/(2*y(4)+y(5));
	% Compute the slopes
   f(:,1) = yderivstwo_new_not(y,KK);
   for j = 1:5
      f(:,j+1) = yderivstwo_new_not(y+h*f*beta(:,j),KK);
   end

        % Update the solution 
      t = t + h;
      y = y + h*f*gamma(:,1);
      k = k+1;
      if k > length(tout)
         tout = [tout; zeros(chunk,1)];
         yout = [yout; zeros(chunk,length(y))];
      end
      tout(k) = t;
      yout(k,:) = y.';

q=q+1;
end;
%Update Graphics
	cfx=-R*y(1);
	cfy=R;
	hipx=cfx-cfhlength*sin(y(1));
	hipy=cfy+cfhlength*cos(y(1));
	kneex=hipx-lt*sin(pi-y(1)-y(2));
	kneey=hipy-lt*cos(pi-y(1)-y(2));
	endx=kneex-(ls-R)*sin(pi-y(1)-y(2)-y(3));
	endy=kneey-(ls-R)*cos(pi-y(1)-y(2)-y(3));


% put end statement down here because we need to calculate end y

walker_data=[cfx,cfy;hipx,hipy;kneex,kneey;endx,endy];
set(walker_bits,'xdata',walker_data(:,1),'ydata',walker_data(:,2));

  drawnow;

%%%%%%%%%%%%%%%% Heelstrike %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
ynew =  heelstrike_turbo(y,m,R,Izz,p12x,p23x,pc1,pc2,pc3,pcs,p1k,etat,ls);

  disp('heelstrike')
  y=ynew
  h=horig/3;
  disp('returning to 3link mode');
  cfx=endx;

%store new y at same t
	 k = k+1;

         if k > length(tout)
                tout = [tout; zeros(chunk,1)];
                yout = [yout; zeros(chunk,length(y))];
         end
         tout(k) = t;
         yout(k,:) = y.';
w=w+1;

step(w,:)=ynew';
%%%%%%%%%%%%%%%%%%%%%%%%% End Main Loop %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  end;
end;

tout = tout(1:k);
yout = yout(1:k,:);
