% run_walker.m file initializes parameters for 2D kneed walker and
% integrates forward to simulate walking down shallow slope (gamma radians)

% Parameters should be entered for thigh(m2) and shank(m3) and
% extrapolated for straight leg combination- modifying this 11/4/94
% to eliminate inconsistent and/or incorrect parameter values

% At kneestrike, the int_walker function calls the function "kneestrike",
% which locks the swing knee and computes new angular velocities
% based on conservation of angular momentum.  Similarly, at heelstrike,
% the function calls "heelstrike" and computes new angular velocities,
% and also re-defines variables so the walker can take a new step.

% walking continues until t=tfinal

% modified 11/6/95 to turbo version to run faster

clf reset
axis equal
clear
format long

%        WORKING SECTION

% THigh
% without extra stuff
wt=0;
lt = 0.35;
ctold = 0.116;
m(2) = 0.9868+0.8;
%m(2) = 0.9868;
emthigh = 0.1718;
%emthigh=0;
cem = 0.029845;

% square masses add 0.1718 at 0.029845 down thigh (calculated 1 thigh)
% adjust thigh to include extra (non-cm) mass added

ct = (ctold*m(2) + cem*emthigh)/(emthigh+m(2));
Izz2noem = 0.0181;
Izz2wem = Izz2noem + emthigh*(cem-ct)^2+m(2)*(ctold-ct)^2;
rt = sqrt(Izz2wem/(m(2)+emthigh));
m(2) = m(2)+emthigh;

mh=0.773/2;
%mh=0;
cth=(ct*m(2))/(m(2)+mh);
Izz2whm=Izz2wem + mh*cth^2 + m(2)*(ct-cth)^2;
m(2)=m(2)+mh;
rt = sqrt(Izz2whm/m(2));
ct=cth;

%Shank
m(3) = 1.013;
rs = sqrt(0.03923/m(3));
ls=0.46;
csold = 0.171;
wsold =0.014;

m(1) = m(2)+m(3);

% slope, foot radius,  and gravity

gam = 0.036;
%gam=0.075;
g = 9.81;
%g=1;
R=0.2;

sgam = sin(gam)*g;
cgam = cos(gam)*g;

%eta values as defined in notes

etak = 0.228;
etat = atan((ls-R)*sin(etak)/((ls-R)*cos(etak)+lt));
etaf = etak-etat;

cs=csold*cos(etak)+wsold*sin(etak);
ws=csold*sin(etak)-wsold*cos(etak);

% fp for labwnks gam=0.036 em=0.8 and 0.1718 mh=0.773
theta1= 0.24395343514010;
theta2 = pi-etat-2*theta1;
theta3 = etak;
theta1dot= -1.04912725993148;
theta2dot= 0.74760219037376;
theta3dot= -1.18165555602257;

%fp for labwnks with m(2)=0.9868 emthigh=0.1718 mh=0.773/2
%theta1= 0.24835996355054;
%theta2 = pi-etat-2*theta1;
%theta3 = etak;
%theta1dot= -1.05868206358140;
%theta2dot= 0.70370523647420;
%theta3dot= -1.08452376169727;

% fp fpr labwnks m(2)=0.9868, mh=0 emthigh=0.1718
%theta1= 0.20383075274100;
%theta2 = pi-etat-2*theta1;
%theta3 = etak;
%theta1dot= -0.96363954597895;
%theta2dot= 0.55349226123587;
%theta3dot= -0.88144104897096;

%fp for labwnks m(2)=0.9868+0.8 mh=0 emthigh=0
%theta1= 0.20357031368894;
%theta2 = pi-etat-2*theta1;
%theta3 = etak;
%theta1dot= -0.94933176929320;
%theta2dot= 0.57856203052525;
%theta3dot= -0.93671332628877;

%fp for labwnks m(2)=0.9868 mh=0 emthigh=0
%theta1= 0.17918654521985;
%theta2 = pi-etat-2*theta1;
%theta3 = etak;
%theta1dot= -0.89897899021611;
%theta2dot= 0.47218071616477;
%theta3dot= -0.76448084225735;

%CLOSE TO A TWO_CYCLE (removed thigh masses)
%fp for labwnks gam=0.036 em=0 and mh=0.773
%theta1= 0.1992792;
%theta2 = pi-etat-2*theta1;
%theta3 = etak;
%theta1dot= -0.956426634;
%theta2dot=  0.54506092;
%theta3dot= -0.8706995;

%gam=0.02152 close to static equilibrium
%theta1=0.019292383921;
%theta2 = pi-etat-2*theta1;
%theta3 = etak;
%theta1dot=-0.0831672026;
%theta2dot=0.00005486;
%theta3dot=-0.00001001;

%gam=0.075; labwalker noknees
%theta1= 0.2824859993335;
%theta2 = pi-etat-2*theta1;
%theta3 = etak;
%theta1dot= -0.4258470102769;
%theta2dot= 0.25835877956118;
%theta3dot= -0.3;


y0 = [theta1 theta2 theta3 theta1dot theta2dot theta3dot];

% vectors from origin of frame to next frame
p01x = R;
p12x = lt*cos(etat)+(ls-R)*cos(etaf);
p23x = lt;

%vectors from origin of frame to cm of leg bit

pc2x = ct;
pc2y = -wt;

pc3x = cs;
pc3y = -ws;

%compute cms of thigh and shank in coordinates of stance leg
%without hip mass

p1c2x = -ct*cos(etat)+wt*sin(etat)+p12x;
p1c2y = ct*sin(etat)+wt*cos(etat);

pcsx = -cs*cos(etaf)-ws*sin(etaf)+(ls-R)*cos(etaf);
pcsy = -cs*sin(etaf)+ws*cos(etaf)+(ls-R)*sin(etaf);
pcs=[pcsx pcsy]';

% compute cm of composite stance leg (thigh+shank)
pc1x = (m(2)*p1c2x+m(3)*pcsx)/m(1);
pc1y = (m(2)*p1c2y+m(3)*pcsy)/m(1);

% coordinates of stance knee from frame 1- used for heelstrike
p1kx = (ls-R)*cos(etaf);
p1ky = (ls-R)*sin(etaf);
p1k = [p1kx p1ky]';

%coordinates of hip from stance knee, used in stance knee torque check
pkct1x = lt*cos(etat);
pkct1y = -lt*sin(etat);

%moments of inertia of leg bits about centers of mass
Izz2 = m(2)*rt^2;
Izz3 = m(3)*rs^2;
Izz1 = Izz2+m(2)*((pc1x-p1c2x)^2+(pc1y-p1c2y)^2)+Izz3+m(3)*...
((pc1x-pcsx)^2+(pc1y-pcsy)^2);

pc2lock(1)=(-pc1x+p12x)*cos(etat)+pc1y*sin(etat);
pc2lock(2)=(-pc1x+p12x)*sin(etat)-pc1y*cos(etat);

Izz = [Izz1 Izz2 Izz3]';
pc1 = [pc1x pc1y]';
pc2 = [pc2x pc2y]';
pc3 = [pc3x pc3y]';

tol = 1e-10;
tfinal =0.9;

global K KK
initializeK_not;
initializeKK;


[t,y,step]=int_walker_turbo(0,tfinal,...
y0,tol,m,g,R,sgam,cgam,Izz,p12x,p23x,pc1,pc2,pc3,pcs,p1k,...
lt,ls,etak,etat,etaf,pc2lock);

y(:,4)=y(:,4)./sqrt(g*(ls+lt));
y(:,5)=y(:,5)./sqrt(g*(ls+lt));
y(:,6)=y(:,6)./sqrt(g*(ls+lt));

%convert to mcgeer output
%y(:,7,8,9) are mcgeer angles and y(:,10,11,12) are corresp angular velocities
y(:,7)=-y(:,1);
y(:,8) = pi-y(:,1)-y(:,2)-etat;
t(:,2)=t(:,1).*(g)^0.5/(ls+lt)^0.5;
y(:,9)= y(:,8)-(y(:,3)-etak);
y(:,10)=-y(:,4);
y(:,11)=-y(:,4)-y(:,5);
y(:,12)=-y(:,4)-y(:,5)-y(:,6);

%plot(t(:,1),y(:,4),t(:,1),y(:,5),t(:,1),y(:,6));
%plot(t(:,2),y(:,10),t(:,2),y(:,11),t(:,2),y(:,12));
plot(t(:,2),y(:,7),t(:,2),y(:,8),t(:,2),y(:,9));
%plot(t(:,1),y(:,1),t(:,1),y(:,2),t(:,1),y(:,3));

vcf1(:,1)=-R.*y(:,4);
vcf1(:,2)=0.*y(:,4)./y(:,4);
vhip(:,1)=vcf1(:,1)-p12x*y(:,4).*cos(y(:,1));
vhip(:,2)=vcf1(:,2)+p12x*y(:,4).*sin(y(:,1));
vknee(:,1)=vhip(:,1)-lt*(-y(:,5)-y(:,4)).*sin(pi-y(:,1)-y(:,2));
vknee(:,2)=vhip(:,2)-lt*(-y(:,5)-y(:,4)).*cos(pi-y(:,1)-y(:,2));
vcf2(:,1)=vknee(:,1)-(ls)*(-y(:,5)-y(:,4)-y(:,6))...
.*sin(pi-y(:,1)-y(:,2)-y(:,3));
vcf2(:,2)=vknee(:,2)-(ls)*(-y(:,5)-y(:,4)-y(:,6))...
.*cos(pi-y(:,1)-y(:,2)-y(:,3));



