function ynew = heelstrike_try(y);
global g gam m R p23 p34 pc3 pc4 p4f Icm3 Icm4 p4f

% Code by Mariano Garcia (msg5@cornell.edu), 1998
% calculates heelstrike velocity jump for 3D walker
% Resets parameters to make L-R mirror image of walker.
% It is called by int_walker_3D.m during forward integration.
% This file is almost identical to heelstrike_mcg.m
% but not quite.

% set sines and cosines and define rotation matrices

cO1 = cos(y(1));
sO1 = sin(y(1));

cO2 = cos(y(2));
sO2 = sin(y(2));

cO3 = cos(y(3));
sO3 = sin(y(3));

sO4 = sin(y(4));
cO4 = cos(y(4));

R01=[cO1 -sO1 0 ;sO1 cO1 0; 0 0 1];
R12=[cO2 -sO2 0 ;0 0 -1;sO2 cO2 0];
R23=[cO3 -sO3 0 ;0 0 -1;sO3 cO3 0];
R34=[cO4 -sO4 0 ;sO4 cO4 0; 0 0 1];

R10=R01';
R21=R12';
R32=R23';
R43=R34';

% At heelstrike, angular momentum is conserved for the whole
% walker about the new contact point and for the swing leg
% about the swing hip. We want to solve for the four angular
% velocities after heelstrike.

% first get angular momentum of whole walker
% about new contact point (cp)

omega1=[0 0 y(5)]';
omega2=R21*omega1+[0 0 y(6)]';
omega3=R32*omega2+[0 0 y(7)]';
omega4=R43*omega3+[0 0 y(8)]';

p233 = R32*p23; % from stance contact to stance center, in frame 2

v3 = scross(omega3,p233); %vel of stance foot center
vc3=scross(omega3,pc3)+v3;
v4 = R43*(scross(omega3,p34)+v3);
vc4 = scross(omega4,pc4)+v4;

rcpc4 = R43*p233-p4f+pc4; % from swing foot contact to cm 4 in frame 4
rcpc3 = p233+R34*(-p4f)-p34+pc3; % from swing foot contact to cm 3 in frame 3

Hcp3 = R34*(cross(rcpc4,m(4)*vc4)+Icm4*omega4)...
+cross(rcpc3,m(3)*vc3)+Icm3*omega3; % H of whole walker about new contact pt

Hcp2 = R23*Hcp3;  % same thing in frame 2
Hcp1 = R12*Hcp2;  % same thing in frame 1

% now get angular momentum of old stance leg about hip in z4-dir

rhc3=-p34+pc3;
Hh3 = cross(rhc3,m(3)*vc3)+Icm3*omega3; % H of swing leg about hip axis

% swap angles for new configuration
% The following is derived from some geometry. See Chapter 2
% of my thesis.

theta3minus=y(3);
y(3)=-pi+y(4)+theta3minus;
y(4)=pi-y(3)+theta3minus; 

% swap z directions for all parameters
pc3(3)=-pc3(3);
p34(3)=-p34(3);
pc4(3)=-pc4(3);

% y(4) = zeroh1(y); %get right on section. Not needed actually,
% since the above algorithm gives an exact solution.

% swap z directions for I matrices
Icm3(1,3)=-Icm3(1,3);
Icm3(2,3)=-Icm3(2,3);
Icm3(3,1)=-Icm3(3,1);
Icm3(3,2)=-Icm3(3,2);
Icm4=Icm3;

% The mass matrix is the matrix of coefficients to get the
% angular momentum just after the impact.

Hpost=getmassmtrx(y);

ydot=Hpost\[Hcp1(3) Hcp2(3) Hcp3(3) Hh3(3)]'; % solve for new angular rates

% reset state vector depending on what driver file is calling this one.
% I dont know if this is necessary any more since it_3D_mcg.m has its
% own heelstrike file.

if length(y) == 10 
ynew=[y(1) y(2) y(3) y(4) ydot(1) ydot(2) ydot(3) ydot(4) 0 0];
end;

if length(y) == 8
ynew=[y(1) y(2) y(3) y(4) ydot(1) ydot(2) ydot(3) ydot(4)];
end;



